function out = pdsnr2(x)
% RFPD SNR maximizer

% Physical Constants (SI)  --------------------
e_c = 1.602e-19;         % electron charge [Coolombs]
kB  = 1.38e-23;          % Boltzmann constant [J/K]
T   = 300;               % room temperature [deg K]
% ---------------------------------------------

% opamp LMH6624 noise at 20 MHz
in = 2.0e-12;
en = 0.9e-9;

% Mod/Demod frequencies
f1 = 36e6; w1=2*pi*f1; %changed from 9e6 RA
f2 = 5 * f1./4; w2=2*pi*f2; %changed from 5*f1 RA
f3 = 2*f2;
L = zeros(size(x));
R = zeros(size(x));
C =  ones(size(x));

% Convert impedance values into component values
for m = 1:length(x)
    if imag(x(m)) > 0
        L(m) = imag(x(m))/w1;
    else
        C(m) = -1/w1/imag(x(m));
    end
    R(m) = real(x(m));
end

% transimpedance and backimpedance
[zT1, zB1] = pdZ(f1, R, L, C);
[zT2, zB2] = pdZ(f2, R, L, C);

% transimpedance and backimpedance at the f1 and f2 readouts
% but at frequency f3
[zT3, zB3] = pdZ(f3, R, L, C);

% shot noise (all SNRs calculated relative to the 1 mA noise equivalent)
Idc = 0.001;
ishot = sqrt(2 * e_c * Idc);


% Shot Noise "Signal" POWER
V11 = abs(zT1(1) * ishot).^2;     % V_f1^2 @ f1 readout
V22 = abs(zT2(2) * ishot).^2;     % V_f2^2 @ f2 readout


% Unwanted signals (V_f1 in the f2 readout, etc.)
V12 = abs(zT1(2) * ishot).^2;     % V_f1^2 @ f2 readout
V21 = abs(zT2(1) * ishot).^2;     % V_f2^2 @ f1 readout
V31 = abs(zT3(1) * ishot).^2;     % V_f3^2 @ f1 readout
V32 = abs(zT3(2) * ishot).^2;     % V_f3^2 @ f2 readout

% Electronics noise POWER (Johnson + voltage + current)
N1 = 4*kB*T*abs(real(zB1(1))) + en^2 + (in * abs(zB1(1))).^2;
N2 = 4*kB*T*abs(real(zB2(2))) + en^2 + (in * abs(zB2(2))).^2;

% power SNRs
SNR1 = abs(V11 ./ N1);
SNR2 = abs(V22 ./ N2);

% final cost function to minimize has 
% equal weighting for both frequencies
alpher = 1; % weighting factor for inverse SNR
bethe  = 100; % weighting factor for signal rejection was 100 RA
out = alpher./SNR1 + 10*alpher./SNR2 + bethe * ((V12./V22 + V21./V11) +...
      (V31./V11 + V32./V22));


