%% Run through many guesses for the RFPD complex impedance values to find the best
N3 = 10e3; %should be a big number (10e3 or more) to adequately search parameter space

% TolX and TolFun termination conditions are 'OR'd to stop the opimization
options = optimset('TolFun',1e-4,...
                   'TolX',1e-5,...
                   'MaxFunEvals',1e5,...
                   'Display','off');

debugnum = 0;

% this freq is used as a scalefactor; exact value not important
w = 2*pi* 36e6; %first of the two RFPD readout frequencies changed from 9e6, RA
w1 = w;
w2 = 5*w./4; %Second readout frequency changed from 5*w

% make initial guess of impedances
%Rinit = 10.^(5 * rand(N3, 5));
%Linit = 3e-10 * 10.^(5 * rand(N3, 5));
%Cinit = 3e-13 * 10.^(7 * rand(N3, 5));
%zinit = i*w*Linit - i./(w*Cinit);

L1 = 1e-3 * rand(N3,1);
C2 = 3e-13 * 10.^(7 * rand(N3, 1));
L3 = 1 ./ (w1^2 * C2);
C4 = 3e-13 * 10.^(7 * rand(N3, 1));
L5 = 1 ./ (w2^2 * C4);
zinit = [1i*w*L1 -1i./(w*C2) 1i*w*L3 -1i./(w*C4) 1i*w*L5];

zarray = zeros(N3, 6);
clear Rinit Linit Cinit L1 C2 L3 C4 L5

%% Run the loop
warning off
tic
%matlabpool
parfor ii = 1:N3
     
            %zzz = fminunc('pdsnr', zinit(ii,:), options);
            %zzz = fminsearch('pdsnr2', zinit(ii,:), options);
            zzz = fminsearch(@(x) pdsnr2(x), zinit(ii,:), options);
            isnr = sqrt(pdsnr2(zzz));
            
            % Check for sanity 
            if (real(zzz) >= 0)                 % non-negative R
              if (-1./(w*imag(zzz)) > 1e-12  |  imag(zzz)./w > 1e-9)  % reasonable L or C
                  if abs(1/isnr) > .1            % SNR OK
                      
                      disp(num2str(1/isnr,3))        % Print score
                      zarray(ii,:) = [zzz 1/isnr];
                  
                  end

              end
            end
            
end
toc
warning on
%clear zinit
%% Print out the results
zarray = sortrows(zarray, -6);   % sort by descending SNR
n = find(zarray(:,6) > 0);
zarray = zarray(n,:);
save zarraydat zarray

%% Convert into R's and C's and L's
if ~exist('zarray'), load zarraydat, end

% Pick the top SNRs
godz = zarray(1:5,:);
ss = size(godz);

% Only print the ones with 'good' values
Rmin = 10;
Cmax = 1e-5;
Lmin = 1e-9;
R = []; C = []; L = [];
vok = 0;  % variable for printing nicely

% loop over solutions
for m = 1:ss(1)
    for n = 1:(ss(2)-1)
        x = real(godz(m,n)); y = imag(godz(m,n));
        if x > Rmin
            disp(['R' num2str(n) ' = ' num2str(x,'%5.1f') ' Ohms'])
            vok = 1;
        end
        if  (y > (w*Lmin))
            disp(['L' num2str(n) ' = ' num2str(y/w*1e6,'%5.2f') ' uH'])  
            vok = 1;
        elseif (y < (-1/(w*Cmax)))
            disp(['C' num2str(n) ' = ' num2str(-1/w/y*1e12,'%5.1f') ' pF'])
             vok = 1;
        end
        
    end
    if (vok), disp(' ----- '), end
    vok = 0;
end
pdZ(logspace(6,8,300),godz(1,1:5),w1/2/pi);

%               Circuit Schematic
%     --------------------------------------
%     |         |        |                  |
%   diode      Z1       Z2                 Z4
%               |        |    f1            |     f2
%              gnd       |-- opamp ->       |-- opamp ->
%                        |                  |
%                       Z3                 Z5
%                        |                  |
%                       gnd                gnd
